const Emitter = require('events');
const Srf = require('../..');
const config = require('config');
//const debug = require('debug')('test:drachtio-srf');

class App extends Emitter {
  constructor() {
    super();

    this.srf = new Srf() ;
    this.srf.connect(config.get('drachtio-sut'));
    this.srf.on('error', (err) => { this.emit('error', err);});
  }

  reject(code, headers) {
    this.srf.invite((req, res) => {
      res.send(code, {headers});
    });
    return this;
  }

  accept(sdp) {
    this.srf.invite((req, res) => {

      req.on('cancel', () => {
        req.canceled = true;
      });
      const localSdp = sdp || req.body.replace(/m=audio\s+(\d+)/, 'm=audio 15000');

      if (req.canceled) return;

      this.srf.createUAS(req, res, {localSdp})
        .then((uas) => {
          this.emit('connected', uas);
          return;
        })
        .catch((err) => {
          console.error(`Uas: failed to connect: ${err}`);
          this.emit('error', err);
        });
    });

    return this;
  }

  acceptCb(callback) {
    this.srf.invite((req, res) => {

      req.on('cancel', () => {
        req.canceled = true;
      });
      const localSdp = req.body.replace(/m=audio\s+(\d+)/, 'm=audio 15000');

      if (req.canceled) return;

      this.srf.createUAS(req, res, {localSdp}, (err, uas) => {
        if (err) {
          console.error(`Uas: failed to connect: ${err}`);
        }
        return;
      });
    });

    return this;
  }

  disconnect() {
    this.srf.disconnect();
    return this;
  }
}

module.exports = App;
